@@ -1,6 +1,15 @@
 """Configuration variables for various types of text.
 """
 
+
+# MESSAGES
+MSG_DATETIME_FORMAT = "%m/%d/%y %H:%M:%S"
+MSG_DAYS_LEN = 3
+MSG_DESCR_LEN = 40
+MSG_ITER_HALF_LEN = 5
+MSG_SEPARATOR = " | "
+
+
 # FILE NAME INDICATORS
 VERSION_INDICATOR = "v"
 
