@@ -2,10 +2,10 @@
 
 import shutil
 
-from eagers.config.path_spec import DEMO_FILES_DIR
+from eagers.config.path_spec import DEMO_FILES_DIR, USER_DIR_LOCATION
 
 
-def load_example_file(filename, cache_location, overwrite_cache=False):
+def load_example_file(filename, cache_name, overwrite_cache=False):
     """Load an example dataset from either the cache or the demo_files
     folder included in the repository.
 
@@ -17,13 +17,15 @@ def load_example_file(filename, cache_location, overwrite_cache=False):
 
     Positional arguments:
     filename - (str) Name of the data set with its file suffix.
-    cache_location - (pathlib.Path) Location of directory to write
-        (or cache) to.
+    cache_name - (str) Name of cache to write to.
 
     Keyword arguments:
     overwrite_cache - (bool) (Default: False) If True, overwrite the
         data set with the file in the demo_files directory.
     """
+    cache_location = USER_DIR_LOCATION.get(cache_name)
+    if not cache_location:
+        raise ValueError(f"Invalid cache name {cache_name!r}")
     cache_path = cache_location / filename
 
     if overwrite_cache or not cache_path.exists():
