@@ -18,17 +18,17 @@ import pathlib
 import numpy as np
 from openpyxl import load_workbook, Workbook
 from openpyxl.drawing.image import Image
-from eagers.class_definition.building import Building
 from eagers.config.dataset import TIMESTAMP_RESOLUTION_TABLENAMES
 from eagers.config.excel_interface import (
     AUTO_STARTDATE_VALUES, HELP_SECTIONHEADER_WIDTH, HELP_TAB_COLOR, STYLES,
     WORKBOOK_SUFFIX)
 from eagers.config.path_spec import (
     EPW_SUFFIX, HDF5_SUFFIX, EXCEL_INTERFACE_PROJECT_HELP,
-    USER_DIR_BUILDINGS, USER_DIR_ENERGYPLUS_WEATHER, USER_DIR_HDF5_DATASETS,
+    USER_DIR_ENERGYPLUS_WEATHER, USER_DIR_HDF5_DATASETS, USER_DIR_LOCATION,
     USER_DIR_PROJECTS, USER_DIR_TESTDATA,
 )
 from eagers.config.text import RUNTIME_HDF5_FILE_PREFIX
+from eagers.extras import bplus_load_building
 from eagers.read.example_file import load_example_file
 from eagers.read.weather_file import read_weather_file
 from eagers.basic.datetime_ext import datetime_from_data
@@ -288,7 +288,7 @@ class BaseTemplateReader(BaseTemplateHandler):
 
     # Subclass properties.
     _example_filename: str
-    _user_dirpath: pathlib.Path
+    _user_cache_name: str
 
     def read(self, filename=None):
         """Read user data from filled in template.
@@ -304,7 +304,7 @@ class BaseTemplateReader(BaseTemplateHandler):
         # https://stackoverflow.com/a/35624928/7232335
         if self.filename == self._example_filename:
             wb = load_workbook(
-                load_example_file(self._example_filename, self._user_dirpath),
+                load_example_file(self._example_filename, self._user_cache_name),
                 read_only=True,
                 data_only=True,
             )
@@ -388,7 +388,7 @@ class ProjectTemplateReader(ProjectTemplateHandler, BaseTemplateReader):
             read_method='_read_optimoptions_sheet',
         ),
     }
-    _user_dirpath = USER_DIR_PROJECTS
+    _user_cache_name = 'projects'
 
     def read(self, *args, **kwargs):
         """Overwrite BaseTemplateReader.read()."""
@@ -443,13 +443,8 @@ class ProjectTemplateReader(ProjectTemplateHandler, BaseTemplateReader):
         buildings = []
         for row in values:
             filename = row[0]
-            try:
-                build = Building.from_file(
-                    load_example_file(filename, USER_DIR_BUILDINGS)
-                )
-            except FileNotFoundError:
-                build = Building.from_file(USER_DIR_BUILDINGS / filename)
-            build.filename = filename
+            build = bplus_load_building(filename)
+            build['filename'] = filename
             buildings.append(build)
         return buildings
 
@@ -600,7 +595,7 @@ class TestDataTemplateReader(TestDataTemplateHandler, BaseTemplateReader):
             read_method='_read_weather_sheet',
         ),
     }
-    _user_dirpath = USER_DIR_TESTDATA
+    _user_cache_name = 'test_data'
 
     def read(self, filename=None):
         """Overwrite BaseTemplateReader.read(). Read user data from
@@ -617,7 +612,7 @@ class TestDataTemplateReader(TestDataTemplateHandler, BaseTemplateReader):
         # https://stackoverflow.com/a/35624928/7232335
         if self.filename == self._example_filename:
             wb = load_workbook(
-                load_example_file(self._example_filename, self._user_dirpath),
+                load_example_file(self._example_filename, self._user_cache_name),
                 read_only=True,
                 data_only=True,
             )
@@ -792,10 +787,7 @@ class TestDataTemplateReader(TestDataTemplateHandler, BaseTemplateReader):
         if filename:
             try:
                 weather_data = read_weather_file(
-                    ensure_suffix(
-                        load_example_file(filename, USER_DIR_ENERGYPLUS_WEATHER),
-                        EPW_SUFFIX,
-                    ),
+                    USER_DIR_LOCATION['eplus_weather'] / ensure_suffix(filename, EPW_SUFFIX),
                     designation,
                 )
             except FileNotFoundError:
