@@ -9,69 +9,79 @@ simulate_response
 """
 
 import datetime
+import warnings
+
+from tables.exceptions import NaturalNameWarning
 
-from eagers.read.excel_interface import ProjectTemplateReader, TestDataTemplateReader
-from eagers.setup.preload import preload
-from eagers.read.read_test_data import read_test_data
-from eagers.read.read_network import read_network
-from eagers.read.read_market import read_market
-from eagers.building.process.load_actual_building_data import load_actual_building_data
-from eagers.building.process.forecast_dr_capacity import forecast_dr_capacity
-from eagers.config.simulation import DEFAULT_INITIAL_SOC
 from eagers.basic.build_time_vector import build_time_vector
 from eagers.basic.get_data import get_data
+from eagers.basic.logger import logger, start_logger
+from eagers.basic.messages import sim_loop_iteration, sim_loop_start
 from eagers.basic.result_template import result_template
 from eagers.basic.auto_set_startdate import auto_set_startdate
-from eagers.simulate.plant_response import plant_response
+from eagers.config.simulation import DEFAULT_INITIAL_SOC
+from eagers.extras import bplus_forecast_dr_capacity, bplus_load_actual_building_data
 from eagers.forecasting.calculate_fits import calculate_fits
 from eagers.forecasting.update_forecast import update_forecast
 from eagers.forecasting.hydro_forecast import hydro_forecast
 from eagers.forecasting.water_year_forecast import water_year_forecast
+from eagers.plot.dispatch_step import new_figure, plot_component_dispatch
+from eagers.read.excel_interface import ProjectTemplateReader, TestDataTemplateReader
+from eagers.read.read_test_data import read_test_data
+from eagers.read.read_network import read_network
+from eagers.read.read_market import read_market
+from eagers.setup.preload import preload
 from eagers.setup.automatic_ic import automatic_ic
 from eagers.setup.initialize_observer import initialize_observer, initialize_building_observer
+from eagers.simulate.plant_response import plant_response
+from eagers.simulate.renewable_output import renewable_output
 from eagers.solver.dispatch_loop import dispatch_loop
 from eagers.solver.generate_bids import generate_bids
 from eagers.update.dispatch_record import dispatch_record, predict_record
-from eagers.simulate.renewable_output import renewable_output
 from eagers.update.update_cost import update_cost
 from eagers.update.update_market import update_market
 from eagers.update.update_observer import update_observer
-from eagers.write.result_file import new_result_file, result_file_dimensions, append_step_solution
-from eagers.plot.dispatch_step import new_figure, plot_component_dispatch
+from eagers.write.result_file import new_result_file, result_file_setup, append_step_solution
 
 
 def demo():
     run_eagers_core('default_project', 'default_testdata', plot_step=True) #Run default project
 
+
 def demo_result():
     from eagers.plot.dispatch_result_excel import plot_dispatch_result
     plot_dispatch_result('default_project', 'default_testdata')
 
-def demo_building():
-    from eagers.building.validate_building import validate_building
-    validate_building(build = 1,climate = '4A', start_day = 1, duration = 10, horizon = 0, mode = 'build',warmup = 'new')
 
-def run_eagers_core(project_filename, testdata_filename, *, plot_step=True):
+def run_eagers_core(
+    project_filename, testdata_filename, *, message_step=True, plot_step=True
+):
     """Load from excel then run a project 
 
         Keyword arguments:
+        message_step - (bool) (Default: True) Whether to output messages
+            at each simulation iteration.
         plot_step - (bool) (Default: True) Whether dispatch should be
             plotted at each simulation iteration.
     """
-    
+    # Filter out NaturalNameWarnings.  https://stackoverflow.com/q/58414068/7232335
+    warnings.filterwarnings("ignore", category=NaturalNameWarning)
     proj = ProjectTemplateReader.read_userfile(project_filename)
     proj['test_data'] = read_test_data(TestDataTemplateReader.read_userfile(testdata_filename))
     read_network(proj['plant']['network']) 
     proj['plant']['market'] = read_market(None)
     if proj['options']['start_date'] is None:
         if len(proj['plant']['building'])>0:
-            proj['options']['start_date'] = proj['plant']['building'][0].sim_date[0]
+            proj['options']['start_date'] = proj['plant']['building'][0]['sim_date'][0]
         else:
             proj['options']['start_date'] = auto_set_startdate(proj['test_data'])
     proj['preload'] = preload(proj['plant'], proj['test_data'] ,proj['options'])
-    run_project(proj, plot_step)
+    if message_step:
+        start_logger(proj['name'])
+    run_project(proj, message_step, plot_step)
 
-def run_project(proj, plot_step):
+
+def run_project(proj, message_step, plot_step):
     """Run the given project. Initializes data structures and then
     passes them on to run_simulation().
 
@@ -84,8 +94,10 @@ def run_project(proj, plot_step):
     date_v = build_time_vector(proj['options']['start_date'], proj['options'], to_timedelta=True)
     
     building_observer,zones,pl = initialize_building_observer(proj['plant']['building'],proj['test_data']['weather'],date)    
-    names, dimensions = result_file_dimensions(proj['preload'],proj['plant'],date_v,zones,pl)
-    new_result_file(proj['name'], names, dimensions, zones, pl)
+    names, dimensions, minimization_directives = result_file_setup(
+        proj['preload'], proj['plant'], date_v, zones, pl
+    )
+    new_result_file(proj['name'], names, dimensions, minimization_directives, zones, pl)
 
     # Create typical day fits if necessary.
     proj['hist_prof'] = calculate_fits(
@@ -109,13 +121,13 @@ def run_project(proj, plot_step):
         market, proj['preload']['subnet'], proj['options'], date_v, proj['test_data'])
     run_simulation(data_t0, date,  proj['test_data'], proj['preload'],
         proj['plant']['building'], fluid_loop, all_data_nodes, names, zones, pl,
-        proj['observer'], market,proj['options'], proj['name'], plot_step)
+        proj['observer'], market,proj['options'], proj['name'], message_step, plot_step)
 
 
 def run_simulation(
         data_t0, date, test_data, preload, building,
         fluid_loop, all_data_nodes, names, zones, pl,
-        observer, market, options, project_name, plot_step):
+        observer, market, options, project_name, message_step, plot_step):
     """Low-level simulation run logic.
 
     Positional arguments:
@@ -133,6 +145,11 @@ def run_simulation(
     # Set up vector of time interval.
     date = build_time_vector(options['start_date'], options, to_timedelta=True)
 
+    # Get start time for console logging.
+    if message_step:
+        t_start = datetime.datetime.now()
+        logger.info(sim_loop_start(project_name, t_start))
+
     # Simulation loop.
     if options['method'] == 'planning':
         num_steps = int(options['interval'] * 24 / (options['resolution']*options['horizon']) + 1)
@@ -155,7 +172,10 @@ def run_simulation(
             forecast['renewable'] = [renewable_output(gen[i], date[1:], forecast['weather']['dir_norm_irr']) \
                                     if gen[i]['type'] in ['Renewable','Solar','Wind'] and gen[i]['enabled'] \
                                     else [] for i in range(len(gen))]
-        forecast['building'], _, _, _ = forecast_dr_capacity(building, observer, forecast['weather'], date)
+        if building:
+            forecast['building'] = bplus_forecast_dr_capacity(building, observer, forecast['weather'], date)
+        else:
+            forecast['building'] = None
         forecast = hydro_forecast(forecast, test_data, date[1:], subnet, observer, options)
 
         solution, _ = dispatch_loop(gen, observer, market, subnet, names, preload['op_mat_a'], 
@@ -187,13 +207,18 @@ def run_simulation(
                 # Stop plotting at every step so the simulation can
                 # continue.
                 plot_step = False
+        if message_step:
+            logger.info(sim_loop_iteration(project_name, t_start, len(timer), num_steps))
 
 
 def load_actual_data(options, observer, test_data, forecast, date, buildings, subnet):
     # Count forward 1 step, rounded to nearest second.
     d_now = [date[0] + datetime.timedelta(hours=options['resolution'])]
     actual_data = get_data(test_data, d_now, subnet['network_names'])
-    actual_data['building'] = load_actual_building_data(buildings,observer,actual_data['weather'],forecast['building'],d_now,options['resolution']*3600)
+    if buildings:
+        actual_data['building'] = bplus_load_actual_building_data(buildings,observer,actual_data['weather'],forecast['building'],d_now,options['resolution']*3600)
+    else:
+        actual_data['building'] = None
     return actual_data
 
 
@@ -209,6 +234,7 @@ def next_prediction(gen,observer,forecast):
         prediction[i].append(forecast[i][-1])
     return prediction
 
+
 def create_fluid_loop(plant):
     fluid_loop = {}
     if len(plant['fluid_loop'])>0:
@@ -222,6 +248,7 @@ def create_fluid_loop(plant):
         fluid_loop['name'] = []
     return fluid_loop
 
+
 def count_nodes(subnet,test_data):
     all_data_nodes = []
     for net in subnet['network_names']:
@@ -238,4 +265,4 @@ def count_nodes(subnet,test_data):
                 if 'demand' in test_data['nodedata_network_info'][net][j]:
                     subnet[net]['load'][n].append(test_data['nodedata_network_info'][net][j]['demand'])
             all_data_nodes.extend(data_nodes)
-    return all_data_nodes
\ No newline at end of file
+    return all_data_nodes
