@@ -176,7 +176,7 @@ def update_mat_costs(qp, gen, margin_cost, scale_cost, ec, dt):
                         #     qp['b'][Rn] = stor_size-buff_size - ec[i] #change lb so that SOC = 0 coresponds to EC          
 
 
-def update_mat_build(qp, observer, fb, dt,):
+def update_mat_build(qp, observer, fb, dt):
     # Within the optimization 5 states are used for a building:
     # #1 represents the air zone temperature setpoint
     # #2 represents the heating required to achieve that zone temperature setpoint
