@@ -1,8 +1,12 @@
 """Defines logic pertaining to the handling of HDF5 simulation results
 files.
 
+Classes:
+Preamble
+
 Functions:
 new_result_file
+result_file_setup
 new_h5_result_filepath
 existing_h5_result_filepath
 append_dispatch_ic
@@ -16,81 +20,135 @@ read_dispatch
 extract_dispatch_data
 """
 
+from typing import Optional
 
 import numpy as np
 
+from eagers.basic.file_handling import (
+    ensure_dirpath,
+    ensure_suffix,
+    find_file_in_userdir,
+)
+from eagers.basic.hdf5 import h5file_context, DatetimeFloatConverter as DFC
 from eagers.config.path_spec import HDF5_SUFFIX, USER_DIR_SIMRESULTS
 from eagers.config.user import DEBUG_MODE
-from eagers.basic.gen_limit import chp_heat
 from eagers.write.result_description import result_description
-from eagers.basic.file_handling import \
-    ensure_dirpath, ensure_suffix, find_file_in_userdir
-from eagers.basic.hdf5 import h5file_context, DatetimeFloatConverter as DFC
 
 
-DICT_ACCESS_PROTOCOL = lambda x, p: x.get(p)
+class Preamble:
+    """Holds a '/'-separated string for defining a path to a certain
+    node in an HDF5 file.
+    """
+
+    sep = "/"
+
+    def __init__(self, s: Optional[str] = None):
+        self.s = s.strip(self.sep) if s else ""
+
+    def __repr__(self):
+        return f"{type(self).__name__}({self.s!r})"
+
+    def __str__(self):
+        return self.s
+
+    def __truediv__(self, other):
+        new_instance = self.__class__(self.s)
+        new_instance.append(other)
+        return new_instance
+
+    def append(self, x):
+        addition = x.strip(self.sep)
+        if self.s:
+            addition = f"{self.sep}{addition}"
+        self.s += addition
+
+    @property
+    def levels(self):
+        return self.s.split(self.sep)
 
 
-def new_result_file(proj_name, names, dimensions, zones, pl):
+def new_result_file(proj_name, names, dimensions, minimization_directives, zones, pl):
     """Create a new HDF5 results file.
 
     Positional arguments:
     proj_name - (str) Name of the project. This becomes the name of the
         file.
-    names - (dict of list of str) all generator, storage, node, line, building and fluid_loop names in
-        the project.
+    names - (dict of list of str) All generator, storage, node, line,
+        building and fluid_loop names in the project.
     dimensions - (dict) Dimension counts for space to be allocated in
         result file structures.
+    minimization_directives - (dict) Directives for minimizing file
+        size.
     """
     # Check that the given file name does not already exist.
     filepath = new_h5_result_filepath(proj_name)
     if not DEBUG_MODE and filepath.exists():
-        raise ValueError(f'File with name {proj_name!r} already exists.')
+        raise ValueError(f"File with name {proj_name!r} already exists.")
 
     # Open file in "w"rite mode.
-    with h5file_context(filepath, mode='w', title=f'{proj_name} results') as h5file:
+    with h5file_context(filepath, mode="w", title=f"{proj_name} results") as h5file:
         # Create tables based on specified dimensions.
-        disp, pred, solu = result_description(names, dimensions, zones, pl)
-        h5file.create_table('/', 'dispatch', disp)
-        h5file.create_table('/', 'predicted', pred)
-        h5file.create_table('/', 'solution', solu)
-      
+        disp, pred, solu = result_description(
+            names, dimensions, minimization_directives, zones, pl
+        )
+        h5file.create_table("/", "dispatch", disp)
+        h5file.create_table("/", "predicted", pred)
+        h5file.create_table("/", "solution", solu)
 
-def result_file_dimensions(preload,plant,date,zones,pl):
+
+def result_file_setup(preload, plant, date, zones, pl):
+    """Returns a dictionary of result file dimensions."""
     # Create dictionary of element names for results to be saved in.
     names = {}
-    names['components'] = [g['name'] for g in preload['gen_qp_form']]
-    names['storage'] = [g['name'] for g in preload['gen_qp_form'] if 'stor' in g]
-    names['nodes'] = [n['name'] for n in plant['network']]
-    names['lines'] = []
-    for net in preload['subnet']['network_names']:
-        snl = preload['subnet'][net]['line']
-        names['lines'].extend([snl['node1'][i]+'_to_'+snl['node2'][i] for i in range(len(snl['node1']))])
-    names['buildings'] = [b.name for b in plant['building']]
-    names['fluid_loop'] = [fl.name for fl in plant['fluid_loop']]
-    if 'district_heat' in preload['subnet']:
-        names['heating_nodes'] = [n[0] for n in preload['subnet']['district_heat']['nodes']]
+    names["components"] = [g["name"] for g in preload["gen_qp_form"]]
+    names["storage"] = [g["name"] for g in preload["gen_qp_form"] if "stor" in g]
+    names["nodes"] = [n["name"] for n in plant["network"]]
+    names["lines"] = []
+    for net in preload["subnet"]["network_names"]:
+        snl = preload["subnet"][net]["line"]
+        names["lines"].extend(
+            [
+                snl["node1"][i] + "_to_" + snl["node2"][i]
+                for i in range(len(snl["node1"]))
+            ]
+        )
+    names["buildings"] = [b["name"] for b in plant["building"]]
+    names["fluid_loop"] = [fl["name"] for fl in plant["fluid_loop"]]
+    if "district_heat" in preload["subnet"]:
+        names["heating_nodes"] = [
+            n[0] for n in preload["subnet"]["district_heat"]["nodes"]
+        ]
     else:
-        names['heating_nodes'] = []
-    if 'district_cool' in preload['subnet']:
-        names['cooling_nodes'] = [n[0] for n in preload['subnet']['district_cool']['nodes']]
+        names["heating_nodes"] = []
+    if "district_cool" in preload["subnet"]:
+        names["cooling_nodes"] = [
+            n[0] for n in preload["subnet"]["district_cool"]["nodes"]
+        ]
     else:
-        names['cooling_nodes'] = []
-    names['hydro'] = [g['name'] for g in preload['gen_qp_form'] if g['type'] == 'HydroStorage' ]
+        names["cooling_nodes"] = []
+    names["hydro"] = [
+        g["name"] for g in preload["gen_qp_form"] if g["type"] == "HydroStorage"
+    ]
     dimensions = dict(
-        horizon = len(date)-1,
-        zone = max(zones),
-        plantloop = max(pl),
-        timer = 3,
+        horizon=len(date) - 1,
+        zone=max(zones),
+        plantloop=max(pl),
+        timer=3,
     )
-    return names, dimensions
+    minimization_directives = dict(
+        heating_exists=bool(names["heating_nodes"]),
+        cooling_exists=bool(names["cooling_nodes"]),
+        hydro_exists=bool(names["hydro"]),
+        multinode=len(names["nodes"]) > 1,
+        fluidloop_exists=bool(names["fluid_loop"]),
+        building_exists=bool(names["buildings"]),
+    )
+    return names, dimensions, minimization_directives
 
 
 def new_h5_result_filepath(filename):
-    """Generate the path to a new HDF5 results file with the given name.
-    """
-    return ensure_dirpath(
-        USER_DIR_SIMRESULTS / ensure_suffix(filename, HDF5_SUFFIX))
+    """Generate the path to a new HDF5 results file with the given name."""
+    return ensure_dirpath(USER_DIR_SIMRESULTS / ensure_suffix(filename, HDF5_SUFFIX))
 
 
 def existing_h5_result_filepath(filename):
@@ -113,11 +171,11 @@ def append_dispatch_ic(proj_name, disp_ic):
     groomed_disp = groom_for_record(disp_ic)
     # Open file in append mode, requiring its existence already.
     filepath = existing_h5_result_filepath(proj_name)
-    with h5file_context(filepath, mode='r+') as h5file:
+    with h5file_context(filepath, mode="r+") as h5file:
+        # Write to the table's Row object, then append it to the table.
         table = h5file.root.dispatch
-        row = table.row
-        write_dispatch_step(row, groomed_disp, DICT_ACCESS_PROTOCOL)
-        row.append()
+        write_dispatch_step(table, groomed_disp)
+        table.row.append()
 
 
 def append_step_predicted(proj_name, pred_step):
@@ -128,17 +186,16 @@ def append_step_predicted(proj_name, pred_step):
     proj_name - (str) Name of the project.
     pred_step - (dict) Data for this step's predicted result.
     """
-    # Convert vanilla Python lists to NumPy arrays, and remove empty lists.
+    # Convert vanilla Python lists to NumPy arrays, and remove empty
+    # lists.
     groomed_pred = groom_for_record(pred_step)
     # Open file in append mode, requiring its existence already.
     filepath = existing_h5_result_filepath(proj_name)
-    with h5file_context(filepath, mode='r+') as h5file:
-        # Get the table's Row object, write to it, then append it to the
-        # table.
+    with h5file_context(filepath, mode="r+") as h5file:
+        # Write to the table's Row object, then append it to the table.
         table = h5file.root.predicted
-        row = table.row
-        write_predicted_step(row, groomed_pred, DICT_ACCESS_PROTOCOL)
-        row.append()
+        write_predicted_step(table, groomed_pred)
+        table.row.append()
 
 
 def append_step_solution(proj_name, solution):
@@ -149,17 +206,16 @@ def append_step_solution(proj_name, solution):
     proj_name - (str) Name of the project.
     solution - (dict) Data for this step's solution result.
     """
-    # Convert vanilla Python lists to NumPy arrays, and remove empty lists.
+    # Convert vanilla Python lists to NumPy arrays, and remove empty
+    # lists.
     groomed_solu = groom_for_record(solution)
     # Open file in append mode, requiring its existence already.
     filepath = existing_h5_result_filepath(proj_name)
-    with h5file_context(filepath, mode='r+') as h5file:
-        # Get the table's Row object, write to it, then append it to the
-        # table.
+    with h5file_context(filepath, mode="r+") as h5file:
+        # Write to the table's Row object, then append it to the table.
         table = h5file.root.solution
-        row = table.row
-        write_solution(row, groomed_solu, DICT_ACCESS_PROTOCOL)
-        row.append()
+        write_solution(table, groomed_solu)
+        table.row.append()
 
 
 def append_step_dispatch(proj_name, disp_step):
@@ -170,17 +226,16 @@ def append_step_dispatch(proj_name, disp_step):
     proj_name - (str) Name of the project.
     disp_step - (dict) Data for this step's dispatch result.
     """
-    # Convert vanilla Python lists to NumPy arrays, and remove empty lists.
+    # Convert vanilla Python lists to NumPy arrays, and remove empty
+    # lists.
     groomed_disp = groom_for_record(disp_step)
     # Open file in append mode, requiring its existence already.
     filepath = existing_h5_result_filepath(proj_name)
-    with h5file_context(filepath, mode='r+') as h5file:
-        # Get the table's Row object, write to it, then append it to the
-        # table.
+    with h5file_context(filepath, mode="r+") as h5file:
+        # Write to the table's Row object, then append it to the table.
         table = h5file.root.dispatch
-        row = table.row
-        write_dispatch_step(row, groomed_disp, DICT_ACCESS_PROTOCOL)
-        row.append()
+        write_dispatch_step(table, groomed_disp)
+        table.row.append()
 
 
 def groom_for_record(data):
@@ -217,89 +272,75 @@ def groom_for_record(data):
     return result
 
 
-def write_dispatch_step(row, disp_step, access):
+def write_dispatch_step(table, disp_step):
     """Write one step of dispatch results to HDF5 file."""
     # Write timestamp.
-    row['timestamp'] = DFC.d2f_sgl2sgl(
-        access(disp_step, 'timestamp'))
-    l = list(disp_step.keys())
-    l.remove('timestamp')
+    table.row["timestamp"] = DFC.d2f_sgl2sgl(disp_step.get("timestamp"))
     # Write dispatch data.
-    for h in l:
-        k_list = list(disp_step[h].keys())
-        if any([isinstance(disp_step[h][k],dict) for k in k_list]):
-            for k in k_list:
-                write_data(row, disp_step, access,h+'/'+k+'/')
-        else:
-            write_data(row, disp_step, access,h+'/')
+    write_data_structure(table, disp_step, Preamble(), {"timestamp"})
 
 
-def write_predicted_step(row, pred_step, access):
+def write_predicted_step(table, pred_step):
     """Write one step of predicted results to HDF5 file."""
     # Write timestamp.
-    row['timestamp'] = DFC.d2f_sgl2sgl(
-        access(pred_step, 'timestamp'))
-    row['cost'] = access(pred_step, 'cost')
-    l = list(pred_step.keys())
-    l.remove('timestamp')
-    l.remove('cost')
+    table.row["timestamp"] = DFC.d2f_sgl2sgl(pred_step.get("timestamp"))
+    table.row["cost"] = pred_step.get("cost")
     # Write predicted data.
-    for h in l:
-        k_list = list(pred_step[h].keys())
-        if any([isinstance(pred_step[h][k],dict) for k in k_list]):
-            for k in k_list:
-                write_data(row, pred_step, access,h+'/'+k+'/')
-        else:
-            write_data(row, pred_step, access,h+'/')
+    write_data_structure(table, pred_step, Preamble(), {"timestamp", "cost"})
 
 
-def write_solution(row, solu, access):
+def write_solution(table, solu):
     """Write solution results to HDF5 file."""
-    row['timestamp'] = DFC.d2f_sgl2sgl(
-        access(solu, 'timestamp'))
-    row['timer'] = access(solu, 'timer')
-    # row['value_heat'] = access(solu, 'value_heat')
-    l = list(solu.keys())
-    l.remove('timestamp')
-    l.remove('timer')
-    l.remove('value_heat')
+    table.row["timestamp"] = DFC.d2f_sgl2sgl(solu.get("timestamp"))
+    table.row["timer"] = solu.get("timer")
     # Write solution data.
-    for h in l:
-        k_list = list(solu[h].keys())
-        if any([isinstance(solu[h][k],dict) for k in k_list]):
-            for k in k_list:
-                write_data(row, solu, access,h+'/'+k+'/')
+    write_data_structure(table, solu, Preamble(), {"timestamp", "timer"})
+
+
+def write_data_structure(table, source, preamble, excluded_fields):
+    keys = set(source.keys()) - excluded_fields
+    for k in keys:
+        v = source.get(k)
+        subitems = None
+        try:
+            subitems = v.items()
+        except (AttributeError, TypeError):
+            # v has no "items" attribute or v.items isn't callable.
+            write_data(table, source, preamble, excluded_fields)
+            continue
+        if any(isinstance(v1, dict) for k1, v1 in subitems):
+            write_data_structure(table, v, preamble / k, excluded_fields)
         else:
-            write_data(row, solu, access,h+'/')
+            write_data(table, source, preamble / k, excluded_fields)
 
 
-def write_data(row, source, access, preamble):
+def write_data(table, source, preamble, excluded_fields):
     """Generalization for writing data from a given set of fields to the
     corresponding HDF5 table location.
 
     Positional arguments:
-    row - (tables.tableextension.Row) Row iterator to write to.
-    source - (object) Object to be read from.
-    access - (function) Access protocol. Must accept two arguments. The
-        first is the object to be accessed. The second is the name of
-        the field to access.
+    table - (tables.Table) Table to write to.
+    source - (dict) Dictionary to be read from.
     preamble - (str) String specifying the path within the HDF5 table to
         the write location.
+    excluded_fields - (set of str) Fields of source to be excluded.
     """
-    # Use specified preamble to get to the part of source we care about.
-    # The trailing '/' exists for consistency between top-level access,
-    # (i.e. preamble = '') and lower-level access, (i.e. preamble =
-    # 'x/.../').  This, along with the [:-1] indexing, ensures we end up
-    # with an empty list in the top-level case and the correct levels in
-    # lower-level cases.
-    levels = preamble.split('/')[:-1]
-    for lvl in levels:
-        source = access(source, lvl)
+    colobjects = table.description
+    for lvl in preamble.levels:
+        try:
+            source = source[lvl]
+        except KeyError:
+            pass
+        colobjects = colobjects._v_colobjects.get(lvl)
 
     # Write data.
-    fields = set(source.keys())
+    # Any fields in the intersection between the description and the
+    # given source will be written to the file.
+    desc_fields = set(colobjects._v_names) if colobjects else set()
+    source_fields = set(source.keys())
+    fields = (desc_fields - excluded_fields) & source_fields
     for f in fields:
-        to_insert = access(source, f)
+        to_insert = source.get(f)
         # Use type check instead of equality check to accommodate NumPy
         # arrays, which return a boolean array when compared to another
         # value.
@@ -307,10 +348,10 @@ def write_data(row, source, access, preamble):
         #   - Raw values (e.g. 3, -2.9, 'electric').
         #   - Non-empty NumPy arrays.
         # Shouldn't encounter a non-empty dict or list here.
-        if not isinstance(to_insert, type(None)) \
-                and (not isinstance(to_insert, (dict, list, np.ndarray))
-                    or len(to_insert)):
-            row[f'{preamble}{f}'] = to_insert
+        if not isinstance(to_insert, type(None)) and (
+            not isinstance(to_insert, (dict, list, np.ndarray)) or len(to_insert)
+        ):
+            table.row[f"{preamble}/{f}"] = to_insert
 
 
 def read_dispatch(project_name, *args):
@@ -352,4 +393,4 @@ def read_dispatch(project_name, *args):
                         unusable = comp['stor']['size'] - comp['stor']['usable_size']
                         st_soc = table.read(field='storage_state/'+k)
                         stor_state[net][k] = [j + unusable for j in st_soc]
-    return gen_state, stor_state
\ No newline at end of file
+    return gen_state, stor_state
